searchData={"items":[{"type":"module","title":"Belodon","doc":"**Belodon** is a utility module designed to process inputs and\ndynamically execute problem-solving functions based on provided modules.\n\nIt supports two flows:\n\n- **Direct Input Flow:** Takes a raw input string along with a part indicator (`:part1` or `:part2`),\nprocesses the input, and then invokes the corresponding `execute/1` function in the submodule.\n- **Auto Input Flow:** Extracts numerical identifiers (typically representing a year and day)\nfrom the module's name, fetches input data via `Belodon.Input.get/3`, and execute the solution.","ref":"Belodon.html"},{"type":"module","title":"Usage - Belodon","doc":"To use this module, create your own module using `mix belodon.create`. Code will be generated in the\ntest folder using the module","ref":"Belodon.html#module-usage"},{"type":"function","title":"Belodon.hello/0","doc":"Hello world.","ref":"Belodon.html#hello/0"},{"type":"function","title":"Examples - Belodon.hello/0","doc":"iex> Belodon.hello()\n    :world","ref":"Belodon.html#hello/0-examples"},{"type":"function","title":"Belodon.solve/3","doc":"Automatically retrieves the input based on the module's name and executes the solution.\n\nThis function is tailored for scenarios where your module's name includes numerical identifiers\n(typically a year and a day). It performs the following steps:\n\n1. **Extract Identifiers:** Uses a regular expression to find numbers in the module name.\n2. **Clean Up:** Trims any leading zeros from the extracted year and day.\n3. **Fetch Input:** Retrieves the problem input using `Belodon.Input.get/3`, passing the day.\n4. **Execution:** Processes the input similar to the direct flow and calls the appropriate solution.","ref":"Belodon.html#solve/3"},{"type":"function","title":"Parameters - Belodon.solve/3","doc":"- `module`: The module containing the solution submodules (`Part1` and `Part2`) with embedded year and day.\n  - `part`: An atom, either `:part1` or `:part2`, to select which solution to run.\n  - `opts`: A keyword list, where each value is passed to the main function and to the file reader.","ref":"Belodon.html#solve/3-parameters"},{"type":"function","title":"Belodon.test/4","doc":"Processes a raw input string and executes the corresponding solution function.\n\nThe process involves:\n\n  1. **Preparation:** The input is split into a list of lines and the last (potentially empty) line is removed.\n  2. **Execution:** Based on the `part` parameter, it dynamically quotes and evaluates code to call\n     the `execute/1` function from the given module's respective nested module (`Part1` or `Part2`).","ref":"Belodon.html#test/4"},{"type":"function","title":"Parameters - Belodon.test/4","doc":"- `module`: The module that contains nested submodules (`Part1` and `Part2`) with an `execute/1` function.\n  - `input`: A multi-line string with the problem input.\n  - `part`: An atom: either `:part1` or `:part2`, which selects the corresponding solution.\n  - `opts`: A keyword list, where each value is passed to the main function.","ref":"Belodon.html#test/4-parameters"},{"type":"module","title":"Belodon.Input","doc":"Module responsible for handling input retrieval and caching\n\nThis module provides facilities to obtain input data for a given year and day by:\n\n  1. **Building a local storage path:** A file path is generated where the input will be cached.\n  2. **Checking for cached input:** If the file exists locally, its contents are returned.\n  3. **Fetching remote input:** If not cached, input is fetched from an external source,\n     its body is processed, and then written locally.","ref":"Belodon.Input.html"},{"type":"module","title":"Workflow overview - Belodon.Input","doc":"graph TD\nA[\"Generate local file path based on year/day\"] --> B[\"If file exists?\"]\nB -- \"No\" --> C[\"Build external input path for fetching\"]\nB -- \"Yes\" --> F\nC --> D[\"Fetch input data\"]\nD --> E[\"Process & cache input locally\"]\nE --> F[\"Process local file & return content\"]","ref":"Belodon.Input.html#module-workflow-overview"},{"type":"function","title":"Belodon.Input.get/3","doc":"Retrieves the input data for a given `year` and `day`.\n\nThis function is the primary entry point for obtaining problem input. It works as follows:\n\n  1. **Local Path Generation:** Computes the local file path.\n  2. **Cache Check:** If the file does not exist, it will:\n       - Build an external path.\n       - Fetch the input\n       - Trim and cache the fetched content locally using `write_file!/2`.\n  3. **Return Value:** Reads and returns the contents of the local file.","ref":"Belodon.Input.html#get/3"},{"type":"function","title":"Parameters - Belodon.Input.get/3","doc":"- `year`: A binary representing the year (e.g., `\"2023\"`).\n  - `day`: A binary representing the day (e.g., `\"15\"`).\n  - `opts`: A keyword list impacting the way data is stored in the file. See [Options](`get/3#options`).","ref":"Belodon.Input.html#get/3-parameters"},{"type":"function","title":"Options - Belodon.Input.get/3","doc":"- `:trim`: Set to `false` if data should not be trimmed (Default: `true`)","ref":"Belodon.Input.html#get/3-options"},{"type":"function","title":"Returns - Belodon.Input.get/3","doc":"- A binary containing the input data.","ref":"Belodon.Input.html#get/3-returns"},{"type":"function","title":"Belodon.Input.write_file!/2","doc":"Writes the given `content` to the file at the specified `path`.\n\nBefore writing, it ensures that the target directory exists by creating it if necessary.","ref":"Belodon.Input.html#write_file!/2"},{"type":"function","title":"Parameters - Belodon.Input.write_file!/2","doc":"- `content`: The content to be written.\n  - `path`: The complete file path where the content should be stored.","ref":"Belodon.Input.html#write_file!/2-parameters"},{"type":"function","title":"Returns - Belodon.Input.write_file!/2","doc":"- `:ok` upon successful file write.","ref":"Belodon.Input.html#write_file!/2-returns"},{"type":"module","title":"Belodon.Types","doc":"Provides custom types and validation functions.\n\nThis module defines custom types for valid years and valid days,\nand offer helper functions to ensure that provided values conform to these ranges.","ref":"Belodon.Types.html"},{"type":"module","title":"Custom Types - Belodon.Types","doc":"- `t:valid_year/0`: An integer between 2022 and 2024 (inclusive)\n  - `t:valid_day/0`: An integer between 1 and 25 (inclusive).\n\nThese types help ensure that year and day values are within the expected bounds.","ref":"Belodon.Types.html#module-custom-types"},{"type":"type","title":"Belodon.Types.valid_day/0","doc":"Represents a valid day for the challenge, an integer between 1 and 25 (inclusive).","ref":"Belodon.Types.html#t:valid_day/0"},{"type":"type","title":"Examples - Belodon.Types.valid_day/0","doc":"iex> is_integer(10) and 1 <= 10 and 10 <= 25\n    true","ref":"Belodon.Types.html#t:valid_day/0-examples"},{"type":"type","title":"Belodon.Types.valid_year/0","doc":"Represents a valid year for the challenge, an integer between 2022 and 2024 (inclusive).","ref":"Belodon.Types.html#t:valid_year/0"},{"type":"type","title":"Examples - Belodon.Types.valid_year/0","doc":"iex> is_integer(2023) and 2022 <= 2023 and 2023 <= 2024\n    true","ref":"Belodon.Types.html#t:valid_year/0-examples"},{"type":"function","title":"Belodon.Types.validate_day!/1","doc":"Validates that the given day is within the allowed range (1 to 25).","ref":"Belodon.Types.html#validate_day!/1"},{"type":"function","title":"Parameters - Belodon.Types.validate_day!/1","doc":"- `day`: An integer representing the day to be validated.","ref":"Belodon.Types.html#validate_day!/1-parameters"},{"type":"function","title":"Returns - Belodon.Types.validate_day!/1","doc":"- The validated day if it is within the accepted range.","ref":"Belodon.Types.html#validate_day!/1-returns"},{"type":"function","title":"Examples - Belodon.Types.validate_day!/1","doc":"iex> Belodon.Types.validate_day!(20)\n    20\n\n    iex> Belodon.Types.validate_day!(30)\n    ** (ArgumentError) Day must be between 1 and 25.","ref":"Belodon.Types.html#validate_day!/1-examples"},{"type":"function","title":"Errors - Belodon.Types.validate_day!/1","doc":"Raises an `ArgumentError` if the provided day is not within the allowed range.","ref":"Belodon.Types.html#validate_day!/1-errors"},{"type":"function","title":"Belodon.Types.validate_module!/1","doc":"Validates that the given module name is not empty and is a string.","ref":"Belodon.Types.html#validate_module!/1"},{"type":"function","title":"Parameters - Belodon.Types.validate_module!/1","doc":"- `module`: The module name to be validated, provided as a string.","ref":"Belodon.Types.html#validate_module!/1-parameters"},{"type":"function","title":"Returns - Belodon.Types.validate_module!/1","doc":"- The validated module name as a string if it is non-empty.","ref":"Belodon.Types.html#validate_module!/1-returns"},{"type":"function","title":"Examples - Belodon.Types.validate_module!/1","doc":"iex> Belodon.Types.validate_module!(\"MyCoolModule\")\n    \"MyCoolModule\"\n\n    iex> Belodon.Types.validate_module!(\"\")\n    ** (ArgumentError) Module must be named.\n\n    iex> Belodon.Types.validate_module!(:my_cool_module)\n    ** (ArgumentError) Module must be a string.\n\n    iex> Belodon.Types.validate_module!(23)\n    ** (ArgumentError) Module must be a string.\n\n    iex> Belodon.Types.validate_module!(true)\n    ** (ArgumentError) Module must be a string.\n\n    iex> Belodon.Types.validate_module!([\"my\", \"cool\", \"module\"])\n    ** (ArgumentError) Module must be a string.\n\n    iex> Belodon.Types.validate_module!({\"my\", \"cool\", \"module\"})\n    ** (ArgumentError) Module must be a string.","ref":"Belodon.Types.html#validate_module!/1-examples"},{"type":"function","title":"Errors - Belodon.Types.validate_module!/1","doc":"Raises an `ArgumentError` if the module name is either empty or not a string.","ref":"Belodon.Types.html#validate_module!/1-errors"},{"type":"function","title":"Belodon.Types.validate_year!/1","doc":"Validates that the given year is within the allowed range (2022 to 2024).","ref":"Belodon.Types.html#validate_year!/1"},{"type":"function","title":"Parameters - Belodon.Types.validate_year!/1","doc":"- `year`: An integer representing the year to be validated.","ref":"Belodon.Types.html#validate_year!/1-parameters"},{"type":"function","title":"Returns - Belodon.Types.validate_year!/1","doc":"- The validated year if it is within the accepted range.","ref":"Belodon.Types.html#validate_year!/1-returns"},{"type":"function","title":"Examples - Belodon.Types.validate_year!/1","doc":"iex> Belodon.Types.validate_year!(2024)\n    2024\n\n    iex> Belodon.Types.validate_year!(2025)\n    ** (ArgumentError) Year must be between 2022 and 2024.","ref":"Belodon.Types.html#validate_year!/1-examples"},{"type":"function","title":"Errors - Belodon.Types.validate_year!/1","doc":"Raises an `ArgumentError` if the provided year is not within the allowed range.","ref":"Belodon.Types.html#validate_year!/1-errors"},{"type":"task","title":"mix belodon.create","doc":"Generate boilerplate solution and test files for a specific puzzle based on\nthe given year, day and module prefix.\n\nThis Mix task creates two files:\n\n  - A solution file under `lib/year /day .ex`, which defines a module\n    for the puzzle's solution.\n  - A test file under `test/year /day _test.exs`, which provides a starting\n    point for testing the solution.","ref":"Mix.Tasks.Belodon.Create.html"},{"type":"task","title":"Command-Line parameters - mix belodon.create","doc":"- `--year` (`-y`): Specifies the target year for the puzzle.\n    Defaults to the environment variable `BELODON_TEST_YEAR`.\n  - `--day` (`-d`): Specifies the target day for the puzzle.\n    Defaults to the environment variable `BELODON_TEST_DAY`.\n  - `--module` (`-m`): Specifies the module prefix to be used for the generated modules.\n    Defaults to the environment variable `BELODON_TEST_MODULE`.","ref":"Mix.Tasks.Belodon.Create.html#module-command-line-parameters"},{"type":"task","title":"Examples - mix belodon.create","doc":"To create files for the puzzle corresponding to year `2024` and day `25` with\na module prefix of `MyCoolModule`, you can run:\n\n````shell\n$ mix belodon.create --year 2024 --day 25 --module MyCoolModule\n# Alternate invocation using short options\n$ mix belodon.create -y 2024 -d 25 -m MyCoolModule\n````\n\nThis will generate:\n\n  - A solution file: `lib/year2024/day25.ex` containing the module `MyCoolModule.Year2024.Day25`.\n  - A test file: `lib/year2024/day25_test.exs` containing the tests for the solution.","ref":"Mix.Tasks.Belodon.Create.html#module-examples"},{"type":"task","title":"mix belodon.test","doc":"Execute tests for a specific puzzle based on the provided year and day.\n\nThis task runs tests for a puzzle by determining the appropriate test file path.\nIt validates the year and day using the function from `Belodon.Types`, ensuring\nthey fall within the allowed ranges.\nIf parameters via the command line, the task defaults to the environment variables\n`BELODON_TEST_YEAR` and `BELODON_TEST_DAY`.","ref":"Mix.Tasks.Belodon.Test.html"},{"type":"task","title":"Command-Line parameters - mix belodon.test","doc":"- `--year` (`-y`): Target year for the puzzle.\n  Defaults to the value of the environment variable `BELODON_TEST_YEAR`.\n  - `--day` (`-d`): Target day for the puzzle.\n  Default to the value of the environment variable `BELODON_TEST_DAY`.","ref":"Mix.Tasks.Belodon.Test.html#module-command-line-parameters"},{"type":"task","title":"Examples - mix belodon.test","doc":"````shell\n# Runs tests for the puzzle from year 2024, day 25\n$ mix belodon.test --year 2024 --day 25\n# Alternate invocation using short options\n$ mix belodon. test -y 2024 -d 25\n````\n\nThe task constructs the test file path in the format\n`test/year /day _test.exs` and then execute the tests.","ref":"Mix.Tasks.Belodon.Test.html#module-examples"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.37.3"}}